

//gcc -Wall -w -O2 -DUSE_DOUBLE -I/opt/csound6-git -I/opt/csound6-git/include -I/usr/include/glib-2.0 -I/usr/lib64/glib-2.0/include  -lglib-2.0 -o slist_ex slist_ex.c -L. -lcsound64 -lsndfile -ldl -lm -lpthread -lfftw3

/* A while loop accessess a glib slist of numbers, assigns it to a struct int,
* then triggers a csound instrument with that number
* using csoundGetChannelPtr in a separate thread.
* It uses csoundSleep to pause the threads between instrument numbers*/

#include <stdio.h> 
#include <stdlib.h>
#include <csound.h> 
#include <glib.h>

uintptr_t csThread(void *clientData); 

GSList *number_list = NULL;

typedef struct { 
int result; 
CSOUND *csound; 
int PERF_STATUS; 
int myinst;
} userData; 

void efficient_append(GSList** list, GSList** list_end, gpointer data)
{
g_return_if_fail(list != NULL);
g_return_if_fail(list_end != NULL);

if (*list == NULL)
{
g_assert(*list_end == NULL);

*list = g_slist_append(*list, data);
*list_end = *list;     
}
else 
{
*list_end = g_slist_append(*list_end, data)->next;
}
} 

int main(void) 
{
int i;
void *ThreadID; 
void *JsThreadID; 
userData *ud; 
ud = (userData *)malloc(sizeof(userData));  
const gchar * glib_check_version ();
const char *argv[] = {"csound", "-+rtaudio=ALSA", "-odevaudio", "-s", "-d", "-b1024", "-B1024","slist_ex.csd" };

GSList* list_end = g_slist_last(number_list);

//loads the list using efficient_append func, above
for (i = 2; i < 14; i++)
{
efficient_append(&number_list, &list_end, GINT_TO_POINTER (i));
//printf (" %u", k);
}

while (number_list != NULL)
{ 
//assigns list number using slist struct pointer to data
ud->myinst = GPOINTER_TO_INT(number_list->data);
printf("\n ****** Struct data, the list number is: %d *******\n\n", ud->myinst);

//create a new Csound for each time through the while loop
ud->csound = csoundCreate(NULL);  
ud->result = csoundCompile(ud->csound, 8, (char **)argv);
ud->PERF_STATUS = 1; 
//create a csound thread
ThreadID = csoundCreateThread(csThread, (void *)ud); 
JsThreadID = csoundGetCurrentThreadId();
//print the thread ID 
printf( "\n\n ***** DEBUG: csoundGetCurrentThreadId = %d \n\n", JsThreadID);
//pause not needed here, but it helps
csoundSleep(2000);  //each inst plays for 1 sec., csoundSleep pauses for 2 sec. between threads.
}  //end while

//free memory
g_slist_free(number_list);
ud->PERF_STATUS = 0; 
csoundDestroy(ud->csound); 
free(ud); 
return 0; 
}  //end main

/* performance thread function */
uintptr_t csThread(void *data) 
{ 
MYFLT *pvalue;
userData *udata = (userData *)data; 

if (!udata->result) {
//printf( "\n\n ***** DEBUG: udata->myinst, from csThread = %d \n\n", udata->myinst);

while ((csoundPerformKsmps(udata->csound) == 0) &&
(udata->PERF_STATUS == 1))

if (csoundGetChannelPtr(udata->csound, &pvalue, "inst",
CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0)
{
*pvalue = udata->myinst;
}
}  //end if loop
csoundDestroy(udata->csound);       
udata->PERF_STATUS = 0; 
//need to increment here after the thread is done
//or could add a callback in main with if true then increment
number_list = g_slist_next(number_list);    
return 1; 
}  

/* REFERENCES
* floss manual Csound API template:
* http://files.csound-tutorial.net/floss_manual/Release03/Cs_FM_03_ScrapBook/the-csound-api.html
*  
* efficient_append function from gtk+
* http://www.linuxtopia.org/online_books/gui_toolkit_guides/gtk+_gnome_application_development/z29.html
*/

//jhearon, 2016
