

//gcc -Wall -w -O2 -DUSE_DOUBLE -I/opt/csound6-git -I/opt/csound6-git/include -I/usr/include/glib-2.0 -I/usr/lib64/glib-2.0/include  -lglib-2.0 -o list_ex list_ex.c -L. -lcsound64 -lsndfile -ldl -lm -lpthread -lfftw3

/* Load the lists and then plays either forward or reverse.
* A while loop accessess the glib list of numbers, assigns it to a struct int,
* then triggers a csound instrument with that number
* using csoundGetChannelPtr in a separate thread.
* It uses csoundSleep to pause the threads between instrument numbers*/

#include <stdio.h> 
#include <stdlib.h>
#include <csound.h> 
#include <glib.h>

uintptr_t csThread(void *clientData); 

GList *number_list = NULL;
GList *revnumber_list = NULL;
GList *forwardnumber_list = NULL;

typedef struct { 
int result; 
CSOUND *csound; 
int PERF_STATUS; 
int myinst;
} userData; 

int main(void) 
{
int i;
char c;
void *ThreadID; 
void *JsThreadID; 
userData *ud; 
ud = (userData *)malloc(sizeof(userData));  
const gchar * glib_check_version ();
const char *argv[] = {"csound", "-+rtaudio=ALSA", "-odevaudio", "-s", "-d", "-b1024", "-B1024","list_ex.csd" };

//load the lists
for (i = 2; i < 14; i++) {
number_list = g_list_append (number_list, GINT_TO_POINTER (i));
forwardnumber_list = g_list_append (forwardnumber_list, GINT_TO_POINTER (i));
revnumber_list = g_list_prepend (revnumber_list, GINT_TO_POINTER (i));
}

//****************************************
//********  User Switch to Play forward/reverse, or quit *****
//****************************************
userswitch:
printf("\n");
printf("\nPLay? (f)orward, (r)everse or (q)uit: ");
scanf("%c", &c);
while (getchar()!='\n');
switch(c)
{
case 'f':
printf("\n f selected");
number_list = forwardnumber_list;
goto next;
case 'r':
printf("\n r selected");
number_list = revnumber_list;
goto next;

case 'q':
goto last;

default:
break;
} //ends switch


next:
while (number_list != NULL)
{ 
//assigns list number using list struct pointer to data
ud->myinst = GPOINTER_TO_INT(number_list->data);
printf("\n *** Struct data, the list number is: %d ****\n", ud->myinst);

//create a new Csound for each time through the while loop  
ud->csound = csoundCreate(NULL);  
ud->result = csoundCompile(ud->csound, 8, (char **)argv);
ud->PERF_STATUS = 1; 
//create a csound thread
ThreadID = csoundCreateThread(csThread, (void *)ud); 
JsThreadID = csoundGetCurrentThreadId();
//print the thread ID 
printf( "\n\n ***** DEBUG: csoundGetCurrentThreadId = %d \n\n", JsThreadID);
//pause not needed here, but it helps
csoundSleep(2000);  //each inst plays for 1 sec., csoundSleep pauses for 2 sec. between threads.
}  //end while

goto userswitch;

last:
printf("\nbye");
//free memory
g_list_free(number_list);
g_list_free(forwardnumber_list);
g_list_free(revnumber_list);
ud->PERF_STATUS = 0; 
csoundDestroy(ud->csound); 
free(ud); 
return 0; 
} 


/* performance thread function */
uintptr_t csThread(void *data) 
{ 
MYFLT *pvalue;
userData *udata = (userData *)data; 

if (!udata->result) {
//printf( "\n\n ***** DEBUG: udata->myinst, from csThread = %d \n\n", udata->myinst);

while ((csoundPerformKsmps(udata->csound) == 0) &&
(udata->PERF_STATUS == 1))

if (csoundGetChannelPtr(udata->csound, &pvalue, "inst",
CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0)
{
*pvalue = udata->myinst;
}

}  //end if loop
csoundDestroy(udata->csound);       
udata->PERF_STATUS = 0; 

//has to increment here after the thread is done
//or could add a callback in main with if true then increment
number_list = g_list_next(number_list);    
return 1; 
}  


/* REFERENCES
* floss manual Csound API template:
* http://files.csound-tutorial.net/floss_manual/Release03/Cs_FM_03_ScrapBook/the-csound-api.html
*/


//jhearon, 2016
