
//gcc -Wall -w -O2 -DUSE_DOUBLE -I/opt/csound6-git -I/opt/csound6-git/include -I/usr/include/glib-2.0 -I/usr/lib64/glib-2.0/include  -lglib-2.0 -o hashtable_ex hashtable_ex.c -L. -lcsound64 -lsndfile -ldl -lm -lpthread -lfftw3

/* a seqeuence of instrument numbers is entered and given a hash value
* those are passed a userdata to the csound thread and the csound inst
* is performed using the inst number and it prints the num and hash value or name
* while it plays
*/

#include <stdio.h> 
#include <stdlib.h>
#include <string.h>
#include <csound.h> 
#include <glib.h>

uintptr_t csThread(void *clientData); 

typedef struct { 
int result; 
CSOUND *csound; 
int PERF_STATUS; 
int myinst;
char* somevalue;
} userData; 

/** This prints out a key/value combination, for use in "g_hash_foreach".
**/
void printKeyValue( gpointer key, gpointer value, gpointer userData ) {
int realKey = GPOINTER_TO_INT( key );
char* realValue = (char*)value;
printf( "%d => %s\n", realKey, realValue );
return;
}

/** The main function.
**/
int main(void) {
int i;
void *ThreadID; 
void *JsThreadID; 
userData *ud; 
ud = (userData *)malloc(sizeof(userData));  
const gchar * glib_check_version ();
const char *argv[] = {"csound", "-+rtaudio=ALSA", "-odevaudio", "-s", "-d", "-b1024", "-B1024","hashtable_ex.csd" };

//assign a pointer to hashtable
GHashTable* map = NULL;
//a pointer to value for data.
char* value = NULL;
//create an iterator
GHashTableIter iter;
//pointer to key and value
gpointer mykey, myvalue;

// Create the GHashTable.
map = g_hash_table_new_full(
g_direct_hash, g_direct_equal, //< This is an integer hash.
NULL, //< There is nothing to free for our key.
free //< Call "free" on the value (made with "strdup").
);

// Insert some values.
// 2 => sin
value = strdup( "sin" );
g_hash_table_insert( map, GINT_TO_POINTER( 2 ), value );

// 3 => sinphase
value = strdup( "sinphase" );
g_hash_table_insert( map, GINT_TO_POINTER( 3 ), value );

// 4 => rsplinesin
value = strdup( "rsplinesin" );
g_hash_table_insert( map, GINT_TO_POINTER( 4 ), value );

// 5 => rsplinecos
value = strdup( "rsplinecos" );
g_hash_table_insert( map, GINT_TO_POINTER( 5 ), value );

// 6 => rsplinecosphase
value = strdup( "rsplinecosphase" );
g_hash_table_insert( map, GINT_TO_POINTER( 6 ), value );

// 7 => rsplinesinphase
value = strdup( "rsplinesinphase" );
g_hash_table_insert( map, GINT_TO_POINTER( 7 ), value );

// 8 => gaussian
value = strdup( "gaussian" );
g_hash_table_insert( map, GINT_TO_POINTER( 8 ), value );

// 9 => gaussianenvel
value = strdup( "gaussianenvel" );
g_hash_table_insert( map, GINT_TO_POINTER( 9), value );

// 10 => rsplinenoise
value = strdup( "rsplinenoise" );
g_hash_table_insert( map, GINT_TO_POINTER( 10 ), value );

// 11 => normalize
value = strdup( "normalize" );
g_hash_table_insert( map, GINT_TO_POINTER(11 ), value );

// 12 => sawtoothsin
value = strdup( "sawtoothsin" );
g_hash_table_insert( map, GINT_TO_POINTER( 12 ), value );

// 13 => sawtoothcos
value = strdup( "sawtoothcos" );
g_hash_table_insert( map, GINT_TO_POINTER( 13 ), value );

// Print out the contents of the map.
g_hash_table_foreach( map, printKeyValue, NULL );

//iterates the keys and values of the hashtable  
g_hash_table_iter_init (&iter, map);              
while (g_hash_table_iter_next (&iter, &mykey, &myvalue))
{
//assigns number to pointer myvalue and key to mykey
ud->myinst = mykey;
ud->somevalue = myvalue;
printf("\n *** Struct data, the list number is: %d  and value is: %s ****\n\n", ud->myinst, ud->somevalue);
//create a new Csound for each time through the iterator while loop 
ud->csound = csoundCreate(NULL);  
ud->result = csoundCompile(ud->csound, 8, (char **)argv);
ud->PERF_STATUS = 1; 
//create a csound thread
ThreadID = csoundCreateThread(csThread, (void *)ud); 
JsThreadID = csoundGetCurrentThreadId();
//print the thread ID 
printf( "\n\n ***** DEBUG: csoundGetCurrentThreadId = %d \n\n", JsThreadID);
//pause not needed here, but it helps
csoundSleep(2000);  //each inst plays for 1 sec., csoundSleep pauses for 2 sec. between threads.
}  //end while

// Destroy the map.
g_hash_table_destroy( map );
map = NULL;
ud->PERF_STATUS = 0; 
csoundDestroy(ud->csound); 
free(ud); 
return 0; 
} //end main

/* performance thread function */
uintptr_t csThread(void *data) 
{ 
MYFLT *pvalue;
userData *udata = (userData *)data; 

if (!udata->result) {
//printf( "\n\n ***** DEBUG: udata->myinst, from csThread, inst: %d  value: %s\n\n", udata->myinst, udata->somevalue);  
while ((csoundPerformKsmps(udata->csound) == 0) &&
(udata->PERF_STATUS == 1))

if (csoundGetChannelPtr(udata->csound, &pvalue, "inst",
CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0)
{
*pvalue = udata->myinst;
}

}  //end if loop
csoundDestroy(udata->csound);       
udata->PERF_STATUS = 0; 

return 1; 
}  

/* REFERENCES
* printKeyValue, and g_direct_hash, strdup concepts:
* http://blog.sensecodons.com/2012/01/glib-ghashtable-and-gdirecthash.html
*/


//jhearon, 2016
