

//gcc -Wall -w -O2 -DUSE_DOUBLE -I/opt/csound6-git -I/opt/csound6-git/include -I/usr/include/glib-2.0 -I/usr/lib64/glib-2.0/include  -lglib-2.0 -o mybtree_ex mybtree_ex.c -L. -lcsound64 -lsndfile -ldl -lm -lpthread -lfftw3


/* numbers are inserted into a btree out of order, but the tree will balance itself.
 * The btree is traversed and 'g_tree_lookup' accesses the data, in order for
 * Csound performance of the instruments in the .csd
*/

#include <stdio.h> 
#include <stdlib.h>
#include <string.h>
#include <csound.h> 
#include <glib.h>


uintptr_t csThread(void *clientData); 

typedef struct { 
int result; 
CSOUND *csound; 
int PERF_STATUS; 
int myinst;
} userData; 

/*
* CompareNames
*
* Compare function used so sort the tree
*/
gint CompareNames (gconstpointer item1, gconstpointer item2)
{
return *(int*)item1 - *(int*)item2;
}

/* Manually insert elements into balanced btree */
gboolean balanced_btree_insert(GTree* tree, int key, int value){
int * j  =(int *)malloc(sizeof(int));
*j = key;

int * k  =(int *)malloc(sizeof(int));
*k = value;

//checking if element already exists
if(!(g_tree_lookup(tree, (gpointer)j))){
g_tree_insert(tree, (gpointer)j, (gpointer)k);

return TRUE;
}
else return FALSE;
}

/*
* TraverseTree
*
* Function that displays the information at the
* node.
*/
gint TraverseTree (gpointer key, gpointer value, gpointer data)
{
int	* j = key;
int * k = value;

g_print ("Key: %d  Value: %d\n", *j, *k);
return FALSE;
}


/** The main function.
**/
int main(void) {
int i;
void *ThreadID; 
void *JsThreadID; 
userData *ud; 
ud = (userData *)malloc(sizeof(userData));  
const gchar * glib_check_version ();
const char *argv[] = {"csound", "-+rtaudio=ALSA", "-odevaudio", "-s", "-d", "-b1024", "-B1024","btree_ex.csd" };

int * key  =(int *)malloc(sizeof(int));
int * value  =(int *)malloc(sizeof(int));

//declare tree pointer
GTree	*tree;
//create a new binary tree
tree = g_tree_new (CompareNames);

//items inserted out of order, but will balance itself and print in order
balanced_btree_insert (tree, 1, 2);
balanced_btree_insert (tree, 8, 9);
balanced_btree_insert (tree, 4, 5);
balanced_btree_insert (tree, 7, 8);
balanced_btree_insert (tree, 9, 10);
balanced_btree_insert (tree, 2, 3);
balanced_btree_insert (tree, 6, 7);
balanced_btree_insert (tree, 11, 12);
balanced_btree_insert (tree, 5, 6);
balanced_btree_insert (tree, 10, 11);
balanced_btree_insert (tree, 12, 13);
balanced_btree_insert (tree, 3, 4);

//print info
g_print ("Tree height: %d\n", g_tree_height (tree));
g_print ("Tree nodes: %d\n", g_tree_nnodes (tree));

//traverse the tree and get a value
g_print ("Tree:\n ");
g_tree_foreach (tree, TraverseTree, NULL);

for (i=1; i<13; i++){
*key = i;
value = (gintptr) g_tree_lookup (tree, key);
if (!value)  
{  
g_print("%d not found in tree.\n", *key);  
} else {  
g_print("\nLooking up key: %d; found-value: %d\n", *key, *value);  

//get the value from the tree using the key
value = (gintptr) g_tree_lookup (tree, key);
//assigns the key to userData
ud->myinst = *value;
printf("\n *** Struct data, the value is: %d ****\n\n", ud->myinst);

//create a new Csound for each time through the tree traversal
ud->csound = csoundCreate(NULL);  
ud->result = csoundCompile(ud->csound, 8, (char **)argv);
ud->PERF_STATUS = 1; 
//create a csound thread
ThreadID = csoundCreateThread(csThread, (void *)ud); 
JsThreadID = csoundGetCurrentThreadId();
//print the thread ID 
printf( "\n\n ***** DEBUG: csoundGetCurrentThreadId = %d \n\n", JsThreadID);
//pause not needed here, but it helps
csoundSleep(2000);  //each inst plays for 1 sec., csoundSleep pauses for 2 sec. between threads.

} //end if
} //end for

//free memory
g_tree_destroy(tree);
ud->PERF_STATUS = 0; 
csoundDestroy(ud->csound); 
free(ud); 
free(key);
free(value);
return 0; 
} //end main

/* performance thread function */
uintptr_t csThread(void *data) 
{ 
MYFLT *pvalue;
userData *udata = (userData *)data; 

if (!udata->result) {
//printf( "\n\n ***** DEBUG: udata->myinst, from csThread = %d \n\n", udata->myinst);

while ((csoundPerformKsmps(udata->csound) == 0) &&
(udata->PERF_STATUS == 1))

if (csoundGetChannelPtr(udata->csound, &pvalue, "inst",
CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0)
{
*pvalue = udata->myinst;      
}

}  //end if loop
csoundDestroy(udata->csound);       
udata->PERF_STATUS = 0; 

return 1; 
}  



/* REFERENCES
*  GLib source code:
*  glib-master/glib/tests/tree.c
*  
*  https://www.ibm.com/developerworks/linux/tutorials/l-glib/
* 
*/

//jhearon, 2016

