

//gcc -Wall -w -O2 -DUSE_DOUBLE -I/opt/csound6-git -I/opt/csound6-git/include -I/usr/include/glib-2.0 -I/usr/lib64/glib-2.0/include  -lglib-2.0 -o mynarytree_ex mynarytree_ex.c -L. -lcsound64 -lsndfile -ldl -lm -lpthread -lfftw3

/* Numbers, as data, are loaded into the child nodes of the nary tree,
 * and the tree data is accessed using GLib data structure pointers before
 * assigning the number to Csound userData for Csound performance.
 */

#include <stdio.h> 
#include <stdlib.h>
#include <string.h>
#include <csound.h> 
#include <glib.h>

uintptr_t csThread(void *clientData); 

typedef struct { 
int result; 
CSOUND *csound; 
int PERF_STATUS; 
int myinst;
} userData; 

gboolean iter(GNode* n, gpointer data) { 
printf("%d ", GPOINTER_TO_INT(n->data));
return FALSE;
}

/** The main function.
**/
int main(void) {
void *ThreadID; 
void *JsThreadID; 
userData *ud; 
ud = (userData *)malloc(sizeof(userData));  
const gchar * glib_check_version ();
const char *argv[] = {"csound", "-+rtaudio=ALSA", "-odevaudio", "-s", "-d", "-b1024", "-B1024","nary_ex.csd" };

gint j;
gint k = 3;
gpointer mynum;

//Creates a new GNode containing the given data. Used to create the first node in a tree.
GNode* root = g_node_new(GINT_TO_POINTER(1));

//loads the data into the Child Nodes
for (j = 2; j < 14; j++) {
//Inserts a GNode as the last child of the given parent.
g_node_append_data(root, GINT_TO_POINTER(j));
k++;
}

//printing using the interator (above)
printf(">Root and Child nodes\n");
g_node_traverse(root, G_PRE_ORDER, G_TRAVERSE_ALL, -1, iter, NULL);

//control loops to keep csound running and access the child nodes
if (root->children)
{
GNode *child;
child = root->children;

while (child)
{
GNode *current;

current = child;
child = current->next;
mynum = current->data;
//assigns the data to userData
ud->myinst = mynum;
printf("\n *** Child data value is: %d ****\n\n", ud->myinst);
//create a new Csound for each time through the control loop
ud->csound = csoundCreate(NULL);  
ud->result = csoundCompile(ud->csound, 8, (char **)argv);
ud->PERF_STATUS = 1; 
//create a csound thread
ThreadID = csoundCreateThread(csThread, (void *)ud); 
JsThreadID = csoundGetCurrentThreadId();
//print the thread ID 
printf( "\n\n ***** DEBUG: csoundGetCurrentThreadId = %d \n\n", JsThreadID);
//pause not needed here, but it helps
csoundSleep(2000);  //each inst plays for 1 sec., csoundSleep pauses for 2 sec. between threads.

} //end while
} //end if

//free memory
g_node_destroy(root);
ud->PERF_STATUS = 0; 
csoundDestroy(ud->csound); 
free(ud); 
return 0; 
}

/* performance thread function */
uintptr_t csThread(void *data) 
{ 
MYFLT *pvalue;

userData *udata = (userData *)data; 
if (!udata->result) {  
//printf( "\n\n ***** DEBUG: udata->myinst, from csThread = %d \n\n", udata->myinst);

while ((csoundPerformKsmps(udata->csound) == 0) &&
(udata->PERF_STATUS == 1))

if (csoundGetChannelPtr(udata->csound, &pvalue, "inst",
CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0)
{
*pvalue = udata->myinst;
}    
}  //end if loop
csoundDestroy(udata->csound);       
udata->PERF_STATUS = 0; 

return 1; 
}  


/* REFERENCES
*  
*  https://github.com/MarkTseng/mySampleCode/tree/master/anage-C-Data-Using-the-GLib-Collections-example
*  https://sourcecodebrowser.com/glib2.0/2.27.0/gnode_8c.html#aa1f8f4f0a2d6eb0029a40f62fd12a57c
*  https://searchcode.com/codesearch/view/7283799/
*/


//jhearon, 2016
