
//gcc -Wall -w -O2 -DUSE_DOUBLE -I/opt/csound6-git -I/opt/csound6-git/include -I/usr/include/glib-2.0 -I/usr/lib64/glib-2.0/include  -lglib-2.0 -o gseq_ex gseq_ex.c -L. -lcsound64 -lsndfile -ldl -lm -lpthread -lfftw3


/* Numbers are added to a GLib sequence data structure, and 
 * are retrieved using 'g_sequence_get_iter_at_pos' before 
 * assigning them to Csound userData for Csound performance.
 */

#include <stdio.h>
#include <glib.h>
#include <stdlib.h>
#include <string.h>
#include <csound.h> 

GSequence *seq;
GSequenceIter *begin, *end, *mid;

uintptr_t csThread(void *clientData); 

typedef struct { 
int result; 
CSOUND *csound; 
int PERF_STATUS; 
int myinst;
} userData; 

gboolean iter(GNode* n, gpointer data) { 
printf("%d ", GPOINTER_TO_INT(n->data));
return FALSE;
}

static void simple_item(gpointer a)          
{
GSequenceIter *myiter = g_sequence_get_iter_at_pos(seq, GPOINTER_TO_INT(a));
printf(" %d ", GPOINTER_TO_INT(g_sequence_get(myiter))); 
}

gint static simple_int(gpointer a)          
{
GSequenceIter *myiter = g_sequence_get_iter_at_pos(seq, GPOINTER_TO_INT(a));
return(GPOINTER_TO_INT(g_sequence_get(myiter))); 
}

int main(void) {
int i, j;
void *ThreadID; 
void *JsThreadID; 
userData *ud; 
ud = (userData *)malloc(sizeof(userData));  
const gchar * glib_check_version ();
const char *argv[] = {"csound", "-+rtaudio=ALSA", "-odevaudio", "-s", "-d", "-b1024", "-B1024","seq_ex.csd" };

//create a sequence
seq = g_sequence_new (NULL);

//load into the  sequence
for (i = 0; i < 24; i++) {
g_sequence_append (seq, GINT_TO_POINTER(i));
}       

//csound thread loop
for (j = 2; j<14; j++)
{
//assigns the data to userData
ud->myinst = simple_int(GINT_TO_POINTER(j));
printf("\n *** Struct data, the list number is: %d ****\n", ud->myinst);
//create a new Csound for each time through the control loop
ud->csound = csoundCreate(NULL);  
ud->result = csoundCompile(ud->csound, 8, (char **)argv);
ud->PERF_STATUS = 1; 
//create a csound thread
ThreadID = csoundCreateThread(csThread, (void *)ud); 
JsThreadID = csoundGetCurrentThreadId();
//print the thread ID 
printf( "\n\n ***** DEBUG: csoundGetCurrentThreadId = %d \n\n", JsThreadID);
//pause not needed here, but it helps
csoundSleep(2000);  //each inst plays for 1 sec., csoundSleep pauses for 2 sec. between threads.
}  //end for loop

//free memory
g_sequence_free(seq);
ud->PERF_STATUS = 0; 
csoundDestroy(ud->csound); 
free(ud); 
return 0; 
} //end main


/* performance thread function */
uintptr_t csThread(void *data) 
{ 
MYFLT *pvalue;
userData *udata = (userData *)data; 

if (!udata->result) {
//printf( "\n\n ***** DEBUG: udata->myinst, from csThread = %d \n\n", udata->myinst);

while ((csoundPerformKsmps(udata->csound) == 0) &&
(udata->PERF_STATUS == 1))

if (csoundGetChannelPtr(udata->csound, &pvalue, "inst",
CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0)
{
*pvalue = udata->myinst;
}

}  //end if loop
csoundDestroy(udata->csound);       
udata->PERF_STATUS = 0; 

return 1; 
}  


/* REFERENCES
*  
* //https://sourcecodebrowser.com/glib2.0/2.27.3/gsequence_8h.html
*/

//jhearon, 2016
