
//gcc -Wall -w -O2 -DUSE_DOUBLE -I/opt/csound6-git -I/opt/csound6-git/include -I/usr/include/glib-2.0 -I/usr/lib64/glib-2.0/include  -lglib-2.0 -o ghash_slist_ex ghash_slist_ex.c -L. -lcsound64 -lsndfile -ldl -lm -lpthread -lfftw3

/* Using singly linked lists as the value for a hash table
*  which employs keys with characteristic names of musical items:
*  ex. short, long, chords.  Each list has 5 unique integers
*  which trigger instrument numbers with short, and long sounds
*  also chords.
*/

#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <glib/gprintf.h>
#include <csound.h> 

uintptr_t csThread(void *clientData); 
GSList *short_list = NULL;
GSList *long_list = NULL;
GSList *chord_list = NULL;


typedef struct { 
int result; 
CSOUND *csound; 
int PERF_STATUS; 
int myinst;
} userData; 

void add_element(GHashTable* hashtable, GSList *list, gchar * key, gpointer data) {   
list = g_hash_table_lookup(hashtable, key);
list = g_slist_append(list, data);
g_hash_table_insert(hashtable, key, list);
}

void output_content(GHashTable* hashtable, gchar * key) {
GSList * list = g_hash_table_lookup(hashtable, key);
guint length = g_slist_length(list); 
g_printf("length of %s is: %d\n", key, length);

while (list != NULL)
{
printf("List data: %d\n", list->data);
list = g_slist_next(list);
}
}

/** The main function.
**/
int main(void) {

int i;
char c;
void *ThreadID; 
void *JsThreadID; 
userData *ud; 
ud = (userData *)malloc(sizeof(userData));  
const char *argv[] = {"csound", "-+rtaudio=ALSA", "-odevaudio", "-s", "-d", "-b1024", "-B1024","hashlist_ex.csd" };
const gchar * glib_check_version ();

//assign a pointer to hashtable
GHashTable* map = NULL;

gchar *list1 = NULL;
list1 = "shortlist";
gchar *list2 = NULL;
list2 = "longlist";
gchar *list3 = NULL;
list3 = "chordlist";
gpointer mykey, mydata;

//create a new hashtable
map = g_hash_table_new(NULL, NULL);

//load the lists as hashtable values using the gchar as key
for (i = 2; i < 6; i++) {
add_element(map, short_list, list1, GINT_TO_POINTER (i));
}

for (i = 6; i < 10; i++) {
add_element(map, long_list, list2, GINT_TO_POINTER (i));
}

for (i = 10; i < 14; i++) {
add_element(map, chord_list, list3, GINT_TO_POINTER (i));
}

//****************************************
//********  User Switch to choose another list *****
//****************************************
userswitch:
printf("\n");
printf("\nlist (s)hortdurs, (l)ongdurs, (c)hords or (q)uit: ");
//fflush(stdout); 
scanf("%c", &c);
while (getchar()!='\n'); 
switch(c)
{
case 's':
mykey = list1;
goto first;
case 'l':
mykey = list2;
goto first;
case 'c':
mykey = list3;
goto first;
case 'q':
goto last;
break;
default:
break;
} //ends switch
printf("\n");

first:
next:;  //empty statement ';' here to avoid label error

//DEBUG
//output_content(map, mykey);  

GSList * list = g_hash_table_lookup(map, mykey);
guint length = g_slist_length(list); // 
g_printf("length of %s is: %d\n", mykey, length);

while (list != NULL)
{
//assigns the data to userData
ud->myinst = list->data;
printf("\n *** the list value is: %d ****\n", ud->myinst);
//create a new Csound for each time through the control loop
ud->csound = csoundCreate(NULL);  
ud->result = csoundCompile(ud->csound, 8, (char **)argv);
ud->PERF_STATUS = 1; 
//create a csound thread
ThreadID = csoundCreateThread(csThread, (void *)ud); 
JsThreadID = csoundGetCurrentThreadId();
//print the thread ID 
printf( "\n\n ***** DEBUG: csoundGetCurrentThreadId = %d \n\n", JsThreadID);
//pause not needed here, but it helps
csoundSleep(2000);  //each inst plays for 1 sec., csoundSleep pauses for 2 sec. between threads.

list = g_slist_next(list);  //advances the until NULL is reached
} //end while loop

goto userswitch;

last:
printf("\nbye\n");
//clear memory allocations
g_hash_table_destroy( map );
map = NULL;
g_slist_free(short_list);
g_slist_free(long_list);
g_slist_free(chord_list);
ud->PERF_STATUS = 0; 
csoundDestroy(ud->csound); 
free(ud); 
return 0;
} //end main
 

/* performance thread function */
uintptr_t csThread(void *data) 
{ 
MYFLT *pvalue;
userData *udata = (userData *)data; 

if (!udata->result) {
//prints the userdata for the inst number and its hash value or name  
//printf( "\n\n ***** DEBUG: udata->myinst, from csThread, inst: %d  \n\n", udata->myinst);

while ((csoundPerformKsmps(udata->csound) == 0) &&
(udata->PERF_STATUS == 1))

if (csoundGetChannelPtr(udata->csound, &pvalue, "inst",
CSOUND_INPUT_CHANNEL | CSOUND_CONTROL_CHANNEL) == 0)
{
*pvalue = udata->myinst;
}

}  //end if loop
csoundDestroy(udata->csound);       
udata->PERF_STATUS = 0; 

return 1; 
}  

/* REFERENCES
*  
* //http://blog.sensecodons.com/2012/01/glib-ghashtable-and-gdirecthash.html
* //http://gsk.sourceforge.net/gsk-reference/gsk-gskghelpers.html
*/

//jhearon, 2016
